import configparser
import os
from uuid import uuid4


class Config:

    # Config file location
    config_path = None

    # Config
    config = configparser.ConfigParser()

    def __init__(self, config_path):
        self.config_path = config_path

    def get_config(self):
        """
            Will return a user config and set a default if necessary
        """

        # Generate a default config the first time
        if not os.path.isfile(self.config_path):
            self.set_default_config_file()

        # Load existing config
        self.config.read(self.config_path)
        return self.config['MAIN']

    def set_default_config_file(self):
        """
            Set a user default config file
        """

        self.config['MAIN'] = {
            'version': '2.00',
            'keyVersion': '1',  # Will be used to support legacy key versions
                                # if the algorithm changes
            'salt': self.generate_random_salt(),
            'clipboardTTL': '15',
            'hideSecretTTL': '5',
            'autoLockTTL': '900',
            'encryptedDb': True,
        }

        # Save
        self.save_config()

    def update(self, name, value):
        """
            Update a config value
        """

        # Set new value
        self.config['MAIN'][name] = str(value)

        print()
        print('The setting `%s` is now set to `%s`.' % (name, value))
        print()

        # Save
        return self.save_config()

    def save_config(self):
        """
            Save user config to a file
        """

        with open(self.config_path, 'w') as configfile:
            self.config.write(configfile)
        os.chmod(self.config_path, 0o600)

        return True

    def generate_random_salt(self):
        """
            Generate a random salt
            Will be used to generate the vault hash with the user master key
        """

        return str(uuid4())

    def __getattr__(self, name):
        """
            Allows calls to configuration values:
            config = Config()
            print(config.salt) # Will print the salt
        """

        try:
            return self.get_config()[name]
        except KeyError:  # For values that don't exist in the config file
            return None



import unittest
import tempfile
import uuid

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from ..models.base import Base, get_session, get_engine
from ..models.User import UserModel
from ..lib.Config import Config
from ..lib.Encryption import Encryption
from ..modules.carry import global_scope


class BaseTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Set db location
        file_ = tempfile.NamedTemporaryFile(delete=False)
        global_scope['db_file'] = file_.name

        # Create a user key
        cls.secret_key = str(uuid.uuid4())
        cls.enc = global_scope['enc'] = Encryption(cls.secret_key.encode())

        # Load config
        cls.conf_path = tempfile.TemporaryDirectory()
        cls.config = Config(cls.conf_path.name + '/config')
        global_scope['conf'] = cls.config

        # Create engine
        engine = get_engine()

        # Create tables and set database session
        Base.metadata.create_all(engine)
        Session = sessionmaker(bind=engine)
        cls.session = Session()

        # Populate db
        cls.populate_base()

    @classmethod
    def populate_base(cls):
        """
            Populate the database
        """

        # Concatenate key and config's salt
        key_salt = cls.secret_key.encode(
        ) + global_scope['conf'].salt.encode()

        # Save user
        user = UserModel(key='key_validation',
                         value=cls.enc.encrypt(key_salt))
        cls.session.add(user)
        cls.session.commit()

    @classmethod
    def tearDownClass(cls):
        # cls.session.remove()
        pass

        # Cleanup config directory
        cls.conf_path.cleanup()



"""
    See https://stackoverflow.com/questions/3338283/python-sharing-global-variables-between-modules-and-classes-therein/3338357#3338357
    Use  a mutable list to share instances of the same class across multiple modules
"""

global_scope = {
    'enc': None,  # Encryption instance
    'db_file': None,  # Database path
    'conf': None,  # Config instance
}



from unittest.mock import patch

import pyperclip

from ..base import BaseTest
from ...models.Secret import SecretModel
from ...models.Category import CategoryModel
from ...views import secrets
from ...modules.carry import global_scope


class Test(BaseTest):

    def setUp(self):
        # Create some secrets
        secret_1 = SecretModel(name='Paypal',
                               url='https://www.paypal.com',
                               login='gab@gmail.com',
                               password='password123',
                               notes='Some notes',
                               category_id=1)
        self.session.add(secret_1)
        secret_2 = SecretModel(name='Gmail',
                               url='https://www.gmail.com',
                               login='gab@gmail.com',
                               password='password;123',
                               notes='Some notes\nsome more notes')
        self.session.add(secret_2)
        secret_3 = SecretModel(name='eBay',
                               url='https://www.ebay.com',
                               login='gab2@gmail.com',
                               password='123password',
                               notes='')
        self.session.add(secret_3)

        # Add a category as well
        category_1 = CategoryModel(name='My category 1')
        self.session.add(category_1)

        self.session.commit()

    def tearDown(self):
        self.session.query(SecretModel).delete()
        self.session.commit()

    def test_all(self):
        all_secrets = secrets.all()
        self.assertIsInstance(all_secrets, list)
        self.assertEqual(len(all_secrets), 3)

    def test_to_table(self):
        self.assertIsInstance(secrets.to_table(secrets.all()), str)

    def test_to_table_2(self):
        self.assertEqual(secrets.to_table([]), 'Empty!')

    def test_count(self):
        count_secrets = secrets.count()
        self.assertIsInstance(count_secrets, int)
        self.assertEqual(count_secrets, 3)

    def test_get_by_id(self):
        self.assertEqual(secrets.get_by_id(1).name, 'Paypal')

    def test_get_names(self):
        assert secrets.get_names() == ['Paypal', 'Gmail', 'eBay']

    def test_get_top_logins(self):
        assert secrets.get_top_logins() == ['gab@gmail.com', 'gab2@gmail.com']

    def test_add(self):
        self.assertTrue(secrets.add(name='Some name'))

    def test_add_input(self):
        # Leave blank to correctly pass the notes input
        with patch('builtins.input', return_value='1'):
            with patch('getpass.getpass', return_value='some password'):
                self.assertTrue(secrets.add_input())

    def test_add_input_2(self):
        # Simulate user pressing Ctrl-C
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.add_input())

    def test_notes_input(self):
        with patch('builtins.input', return_value='some notes'):
            self.assertEqual(secrets.notes_input(),
                             ('some notes\n' * 15).strip())

    def test_notes_input_2(self):
        with patch('builtins.input', return_value=''):
            self.assertEqual(secrets.notes_input(), '')

    def test_notes_input_3(self):
        # Simulate user pressing Ctrl-C
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.notes_input())

    def test_delete(self):
        # Successful deletion
        self.assertTrue(secrets.delete(1))

    def test_delete_2(self):
        # Deletion of non existing secret
        self.assertFalse(secrets.delete(1234))

    def test_delete_confirm(self):
        # Successful deletion
        with patch('builtins.input', return_value='y'):
            self.assertTrue(secrets.delete_confirm(1))

    def test_delete_confirm_2(self):
        # Test with a non existent ID
        with patch('builtins.input', return_value='y'):
            self.assertFalse(secrets.delete_confirm(1234))

    def test_delete_confirm_3(self):
        # Confirmation denied
        with patch('builtins.input', return_value='n'):
            self.assertFalse(secrets.delete_confirm(1))

    def test_search(self):
        # Search with a name
        results = secrets.search('paypal')
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 1)

    def test_search_2(self):
        # Search with a login
        results = secrets.search('gab@gmail')
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 2)

    def test_search_3(self):
        # Search with a URL
        results = secrets.search('www.gmail.com')
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 1)

    def test_search_dispatch(self):
        # Real int
        results = secrets.search_dispatch(1)
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 1)

    def test_search_dispatch_2(self):
        # String is int
        results = secrets.search_dispatch('1')
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 1)

    def test_search_dispatch_3(self):
        # Integer that does not match an actual ID
        results = secrets.search_dispatch(1234)
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 0)

    def test_search_dispatch_4(self):
        # Keyword
        results = secrets.search_dispatch('ebay')
        self.assertIsInstance(results, list)
        self.assertEqual(len(results), 1)

    def test_search_input(self):
        # Empty search
        with patch('builtins.input', return_value=''):
            self.assertFalse(secrets.search_input())

    def test_search_input_2(self):
        # Fat-finger prevention
        with patch('builtins.input', return_value='q'):
            self.assertEqual(secrets.search_input(), 'q')

    def test_search_input_3(self):
        # Fat-finger prevention
        with patch('builtins.input', return_value='b'):
            self.assertFalse(secrets.search_input())

    @patch.object(secrets, 'item_view')
    def test_search_input_4(self, patched):
        # Search digit
        patched.return_value = None
        with patch('builtins.input', return_value='1'):
            self.assertIsNone(secrets.search_input())

    @patch.object(secrets, 'search_results')
    def test_search_input_5(self, patched):
        # Search string
        patched.return_value = None
        with patch('builtins.input', return_value='@gmail.com'):
            self.assertIsNone(secrets.search_input())

    def test_search_input_6(self):
        # Search with no result
        with patch('builtins.input', return_value='some invalid query'):
            self.assertFalse(secrets.search_input())

    @patch.object(secrets, 'item_view')
    def test_search_results(self, patched):
        # Select valid result
        patched.return_value = None
        results = secrets.all()
        with patch('builtins.input', return_value='1'):
            self.assertIsNone(secrets.search_results(results))

    def test_search_results_2(self):
        # Type integer not corresponding to a result
        results = secrets.all()
        with patch('builtins.input', return_value='1234'):
            self.assertFalse(secrets.search_results(results))

    def test_search_results_3(self):
        # Type a string (invalid input)
        results = secrets.all()
        with patch('builtins.input', return_value='some string'):
            self.assertFalse(secrets.search_results(results))

    def test_search_results_4(self):
        # No input
        results = secrets.all()
        with patch('builtins.input', return_value=''):
            self.assertFalse(secrets.search_results(results))

    @patch.object(secrets, 'item_menu')
    def test_item_view(self, patched):
        patched.return_value = None
        result = secrets.get_by_id(1)
        self.assertIsNone(secrets.item_view(result))

    @patch.object(secrets, 'item_menu')
    def test_item_view_2(self, patched):
        # View item without notes
        patched.return_value = None
        result = secrets.get_by_id(3)
        self.assertIsNone(secrets.item_view(result))

    def test_item_menu(self):
        with patch('builtins.input', return_value='s'):
            self.assertEqual(secrets.item_menu(secrets.get_by_id(1)), 's')

    @patch.object(secrets, 'delete_confirm')
    def test_item_menu_2(self, patched):
        patched.return_value = None

        with patch('builtins.input', return_value='d'):
            self.assertIsNone(secrets.item_menu(secrets.get_by_id(1)))

    def test_item_menu_edit(self):
        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='b'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    def test_item_menu_edit_2(self):
        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value=''):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_3(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='c'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_4(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='n'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_5(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='u'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_6(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='l'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_7(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='p'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    @patch.object(secrets, 'edit_input')
    def test_item_menu_edit_8(self, patched):
        patched.return_value = None

        secret = secrets.get_by_id(1)
        with patch('builtins.input', return_value='o'):
            self.assertIsNone(secrets.item_menu_edit(secret))

    def test_edit_input(self):
        # Edit name
        with patch('builtins.input', return_value='1'):
            self.assertTrue(secrets.edit_input(
                'category', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).category_id, 1)

    def test_edit_input_2(self):
        # Edit category (simulate Ctr-c)
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.edit_input(
                'category', secrets.get_by_id(1)))

    def test_edit_input_3(self):
        # Edit name
        with patch('builtins.input', return_value='new name'):
            self.assertTrue(secrets.edit_input('name', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).name, 'new name')

    def test_edit_input_4(self):
        # Edit name (simulate Ctr-c)
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.edit_input('name', secrets.get_by_id(1)))

    def test_edit_input_5(self):
        # Edit URL
        with patch('builtins.input', return_value='new URL'):
            self.assertTrue(secrets.edit_input('url', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).url, 'new URL')

    def test_edit_input_6(self):
        # Edit URL (simulate Ctr-c)
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.edit_input('url', secrets.get_by_id(1)))

    def test_edit_input_7(self):
        # Edit login
        with patch('builtins.input', return_value='new login'):
            self.assertTrue(secrets.edit_input('login', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).login, 'new login')

    def test_edit_input_8(self):
        # Edit login (simulate Ctr-c)
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.edit_input('login', secrets.get_by_id(1)))

    def test_edit_input_9(self):
        # Edit password
        with patch('getpass.getpass', return_value='new password'):
            self.assertTrue(secrets.edit_input(
                'password', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).password, 'new password')

    def test_edit_input_10(self):
        # Edit password (simulate Ctr-c)
        with patch('getpass.getpass', return_value=False):
            self.assertFalse(secrets.edit_input(
                'password', secrets.get_by_id(1)))

    def test_edit_input_11(self):
        # Edit notes
        with patch('builtins.input', return_value='new notes'):
            self.assertTrue(secrets.edit_input(
                'notes', secrets.get_by_id(1)))

        self.assertEqual(secrets.get_by_id(1).notes,
                         ('new notes\n' * 15).strip())

    def test_edit_input_12(self):
        # Edit notes (simulate Ctr-c)
        with patch('builtins.input', return_value=False):
            self.assertFalse(secrets.edit_input('notes', secrets.get_by_id(1)))

    def test_edit_input_13(self):
        # Edit an invalid column: should raise a ValueError
        self.assertRaises(ValueError, secrets.edit_input,
                          'some invalid value', secrets.get_by_id(1))

    @patch.object(secrets, 'item_view')
    def test_show_secret(self, patched):
        patched.return_value = None

        # Ensure we have a short wait time
        global_scope['conf'].update('hideSecretTTL', '1')

        secret = secrets.get_by_id(1)
        self.assertIsNone(secrets.show_secret(secret))



import os
from hashlib import sha256


import sqlcipher3
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import as_declarative
from sqlalchemy.orm import Session

from ..modules.carry import global_scope


sessions = {}


@as_declarative()
class Base(object):
    """
        Declarative base, shared with all models
    """

    pass


def get_session(fresh=False):
    """
        Return SQLAlchemy session
    """

    global sessions

    if global_scope['db_file'] is None:
        raise RuntimeError('`db_file` is not defined in the global scope')

    # Create a unique key for the db session
    db_file = global_scope['db_file']

    # Add a session to the current list
    if fresh or not sessions.get(db_file):
        sessions[db_file] = Session(bind=get_engine())

    return sessions[db_file]


def drop_sessions():
    """
        Drop current db sessions
    """

    global sessions

    sessions = {}

    return True


def get_engine(encrypted=True):
    """
        return SQLAlchemy engine
    """

    if global_scope['db_file'] is None:
        raise RuntimeError('`db_file` is not defined in the global scope')

    if encrypted:
        return create_engine('sqlite+pysqlcipher://:' + get_db_key() + '@' + get_slashes() + global_scope['db_file'], module=sqlcipher3)
    else:
        return create_engine('sqlite:' + get_slashes() + global_scope['db_file'])


def get_db_key():
    """
        Prepare and return database encryption password
    """

    if global_scope['enc'] is None:
        raise RuntimeError('`enc` is not defined in the global scope')

    if global_scope['conf'] is None:
        raise RuntimeError('`conf` is not defined in the global scope')

    # Retrieve key
    return sha256(global_scope['enc'].key + global_scope['conf'].salt.encode()).hexdigest()


def get_slashes(encrypted=True):
    """
        Return the appropriate number of slash for the database connection
        based on wether the db path is relative or absolute
    """

    if encrypted:
        if os.path.isabs(global_scope['db_file']):
            return '//'

        return '/'
    else:
        if os.path.isabs(global_scope['db_file']):
            return '////'

        return '///'



# Import/export view

import sys
import json

from tabulate import tabulate

from . import menu, secrets, categories
from ..modules.misc import confirm
from ..modules.carry import global_scope
from ..lib.Encryption import Encryption

"""
    Adding import or export formats:

    To add an import or export format, you need to add a file format name to
    `import_()` or `export()` and create an associated method
    called `import_from_[some_format]()` or `export_to[some_format]()`.

    The format name must also be added to `src/vault.py` in argparse choices.

    If you create a format that can be useful to others, please fork the project
    first and submit a merge request!
"""


def import_(format_, path):
    """
        Routing to format specific import methods
    """

    if format_ == 'json':
        return import_from_json(path)
    else:
        raise ValueError('%s is not a supported file format' % (format_))


def export_(format_, path):
    """
        Routing to format specific export methods
    """

    if format_ == 'json':
        return export_to_json(path)
    else:
        raise ValueError('%s is not a supported file format' % (format_))


def export_to_json(path):
    """
        Export to a Json file
    """

    # Ask user to unlock the vault
    unlock()

    # Create dict of secrets
    out = []
    for secret in secrets.all():
        out.append({
            'name': secret.name,
            'url': secret.url,
            'login': secret.login,
            'password': secret.password,
            'notes': secret.notes,
            'category': categories.get_name(secret.category_id),
        })

    return save_file(path, json.dumps(out))


def import_from_json(path=None, rows=None):
    """
        Import a Json file
    """

    # Ask user to unlock the vault (except if its already unlocked in migration)
    if not isinstance(global_scope['enc'], Encryption):
        unlock()

    if not rows:  # If importing from a file
        # Read content
        content = read_file(path)

        # Decode json
        rows = json.loads(content)

    # User view of items
    print("The following items will be imported:")
    print()
    print(to_table(
        [[row['name'], row['url'], row['login'], row['category']] for row in rows]))
    print()

    if confirm('Confirm import?', False):
        return import_items(rows)
    else:
        print("Import cancelled.")
        return False


def import_items(rows):
    """
        Import items at the following format:
        [{'name': '...', 'url': '...', 'login': '...'
        , 'password': '...', 'notes': '...', 'category': '...'}]
    """

    for row in rows:
        # Set category ID
        category_id = None
        if row.get('category'):
            # Search within existing categories
            category_id = categories.get_id(row['category'])

            # Or create a new one
            if category_id is None:
                categories.add(name=row['category'])
                category_id = categories.get_id(row['category'])

        # Create secret
        secrets.add(name=row.get('name'),
                    url=row.get('url'),
                    login=row.get('login'),
                    password=row.get('password'),
                    notes=row.get('notes'),
                    category_id=category_id)

    print('%d items have been imported.' % len(rows))

    return True


def to_table(rows=[]):
    """
        Transform rows in a table
    """

    if len(rows) > 0:
        return tabulate(rows, headers=['Name', 'URL', 'Login', 'Category'])
    else:
        return 'Empty!'


def read_file(path, mode='r'):
    """
        Read an import file and return its content
    """

    # Read import file
    try:
        file = open(path, mode=mode)
        fileContent = file.read()
        file.close()

        return fileContent
    except Exception as e:
        print("The file `%s` could not be opened." % (path))
        print(e)
        sys.exit()


def save_file(path, content, mode='w'):
    """
        Save exported items to a file
    """

    # Save to file
    try:
        file = open(path, mode)
        file.write(content)
        file.close()

        print("The vault has been exported to the file `%s`." % (path))
    except Exception as e:
        print("The vault could not be exported to the file `%s`." % (path))
        print(e)

        return False

    return True


def unlock():
    """
        Ask user to unlock the vault
    """

    # `False` = don't load menu after unlocking
    return menu.unlock(redirect_to_menu=False)



import sys
import os
import tempfile

# Add project root to path so we can import src as a package
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))

from src.lib.Encryption import Encryption
from src.lib.Config import Config
from src.models.Secret import SecretModel
from src.models.Category import CategoryModel
from src.models.User import UserModel
from src.modules.carry import global_scope

def run():
    # Example usage of Encryption class
    print("Starting DynaPyt Driver")
    key = b"supersecretkey"
    enc = Encryption(key)

    global_scope['enc'] = enc
    salt = enc.gen_salt()
    print(f"Generated salt: {salt}")
    digest = enc.digest_key()
    print(f"Digested key: {digest}")
    enc.set_salt()

    # Encrypt and decrypt a message
    message = b"Encrypt This!!"
    print(f"Original message: {message}")
    ciphertext = enc.encrypt(message)
    print(f"Encrypted message (base64): {ciphertext}")
    plaintext = enc.decrypt(ciphertext)
    print(f"Decrypted message: {plaintext}")

    # Encrypt and decrypt with a specific salt
    enc.set_salt(salt)
    secret2 = b"Secret with salt"
    ciphertext2 = enc.encrypt(secret2)
    print(f"Encrypted with salt (base64): {ciphertext2}")
    enc.set_salt(salt)
    plaintext2 = enc.decrypt(ciphertext2)
    print(f"Decrypted with salt: {plaintext2}")

    # Example usage of Config class
    print("\n--- Config Example ---")
    with tempfile.NamedTemporaryFile(delete=False) as tmp:
        config_path = tmp.name

    try:
        conf = Config(config_path)
        conf.set_default_config_file()
        print(f"Config loaded from: {config_path}")
        print(f"Config version: {conf.version}")
        print(f"Config salt: {conf.salt}")

        global_scope['conf'] = conf
        print("\n--- SecretModel Example ---")
        secret = SecretModel("MySecret", "http://example.com", "user", "password123", "my notes")
        print(f"Created Secret: {secret.name}")
        print(f"Secret Login: {secret.login}")
        print(f"Secret Password (decrypted): {secret.password}")
        print(f"Secret Notes (decrypted): {secret.notes}")
        print(f"Internal Encrypted Password: {secret._password}")

    finally:
        if os.path.exists(config_path):
            os.remove(config_path)

    # Example usage of CategoryModel
    print("\n--- CategoryModel Example ---")
    category = CategoryModel(name="Work", active=1)
    print(f"Category: {category}")

    # Example usage of UserModel
    print("\n--- UserModel Example ---")
    user = UserModel(key="setting_key", value="setting_value")
    print(f"User Setting: {user}")

    print("Finished DynaPyt Driver")

if __name__ == "__main__":
    run()



from ..base import BaseTest
from ...models.base import get_session, sessions
from ...models import base
from ...models.User import UserModel
from ...views import users
from ...modules.carry import global_scope
from ...lib.Encryption import Encryption


class Test(BaseTest):

    def setUp(self):
        # Create validation key
        key_salt = global_scope['enc'].key + \
            global_scope['conf'].salt.encode()

        # Save user
        user = UserModel(key='key_validation',
                         value=global_scope['enc'].encrypt(key_salt))
        get_session().add(user)
        get_session().commit()

    def tearDown(self):
        # Truncate table
        self.session.query(UserModel).delete()
        self.session.commit()

    def test_validation_key_new(self):
        users.validation_key_new()

        # Get inserted row
        user = get_session().query(UserModel).filter(
            UserModel.key == 'key_validation').order_by(UserModel.id.desc()).first()

        # Re-create key + salt
        key_salt = global_scope['enc'].key + \
            global_scope['conf'].salt.encode()

        self.assertEqual(global_scope['enc'].decrypt(user.value), (key_salt))

    def test_validation_key_validate(self):
        self.assertTrue(users.validation_key_validate(
            self.secret_key.encode()))

    def test_validation_key_validate_2(self):
        # Testing "except exc.DatabaseError"

        # Force re-initialization of db session
        save_sessions = base.sessions
        base.sessions = {}

        # Force wrong key in Encryption class
        global_scope['enc'].key = b'some invalid key'

        self.assertFalse(users.validation_key_validate(
            b'some invalid key'))

        # Restore db sessions
        base.sessions = save_sessions

        # Restore key
        global_scope['enc'].key = self.secret_key.encode()

    def test_validation_key_validate_3(self):
        # Testing "except ValueError" for key decryption error

        self.assertFalse(users.validation_key_validate(
            b'some invalid key'))

    def test_validation_key_rekey(self):
        enc = Encryption(b'new key')
        self.assertTrue(users.validation_key_rekey(enc))

    def test_validation_key_rekey_2(self):
        # Test without a valid row in the table

        # Truncate table
        self.session.query(UserModel).delete()
        self.session.commit()

        enc = Encryption(b'new key')
        self.assertFalse(users.validation_key_rekey(enc))



from unittest.mock import patch
import uuid

from ..base import BaseTest
from ...views import setup
from ...models.base import get_session
from ...models.User import UserModel
from ...modules.carry import global_scope


class Test(BaseTest):

    def test_setup(self):
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(setup.initialize(self.config.salt))

    def test_create_db(self):
        self.assertTrue(setup.create_db())

    def test_get_key_input(self):
        input_ = str(uuid.uuid4())

        with patch('getpass.getpass', return_value=input_):
            self.assertEqual(setup.get_key_input(), input_)

    def test_get_key_input_2(self):
        with patch('getpass.getpass', return_value='abc'):
            self.assertFalse(setup.get_key_input())

    def test_is_key_valid(self):
        self.assertTrue(setup.is_key_valid(
            'some_valid_key_123'))

    def test_is_key_valid_2(self):
        self.assertFalse(setup.is_key_valid('abc'))

    def test_check_key_and_repeat(self):
        self.assertTrue(setup.check_key_and_repeat(
            'some_valid_key_123', 'some_valid_key_123'))

    def test_check_key_and_repeat_2(self):
        self.assertFalse(setup.check_key_and_repeat(
            'some_valid_key_123', 'some_valid_key_123456'))



# Import/export view

import sys

from sqlalchemy.orm import Session

from . import menu, secrets
from .setup import get_key_input
from ..modules.carry import global_scope
from ..lib.Encryption import Encryption
from .users import validation_key_rekey
from ..models.base import Base, get_session, get_engine
from ..models.Category import CategoryModel  # Imported for schema creation
from ..models.Secret import SecretModel  # Imported for schema creation
from ..models.User import UserModel  # Imported for schema creation

enc_current = enc_new = None


def rekey():
    """
        Change the master key. This involves 3 steps:
         - Looping thru all secrets and encrypting them with the new key
         - Updating the validation key
         - Re-keying the database
    """

    global enc_current, enc_new

    # Ask user to unlock the vault
    print('\n* Please enter your CURRENT master key:')
    unlock()

    # Set instance of current Encryption class
    enc_current = global_scope['enc']

    # Ask the user to enter the new master key
    print('\n* Please enter your NEW master key:\n')
    newkey = get_key_input()

    if newkey:
        # Set instance of new Encryption class
        enc_new = Encryption(newkey.encode())

        # Loop thru all secrets and encrypt them with the new key
        rekey_secrets()

        # Re-key the validation key with the new master key
        rekey_validation_key()

        # Change the db encryption key
        rekey_db()

        print('\nYour master key has been updated.')
        print('You can now re-start the application.')

        return True

    return False


def rekey_secrets():
    """
        Loop thru all secrets and encrypt them with the new key
    """

    for secret in secrets.all():
        # Get current values
        password = secret.password
        notes = secret.notes

        # Set Encryption to new class
        global_scope['enc'] = enc_new

        # Update password, notes
        secret.password = password
        secret.notes = notes

        # Revert Encryption to current class
        global_scope['enc'] = enc_current

        get_session().add(secret)

    get_session().commit()

    return True


def rekey_validation_key():
    """
        Re-key the validation key with the new master key
    """

    return validation_key_rekey(enc_new)


def rekey_db():
    """
        Change the db encryption key
    """

    # # Get engine with rekey
    # engine = get_engine(add_to_connection_string='?rekey=' +
    #                     enc_new.key.decode('utf-8'))

    # # Create new session
    # session = Session(bind=engine)
    # session.commit()

    # # Update global scope
    # global_scope['enc'] = enc_new

    print('Change the db encryption key: not implemented!')

    return None


# def new_db_get_engine():
#     """
#         Will return a SQLite engine with the new key and a temporary location
#     """

#     # Save db file
#     db_file = global_scope['db_file']

#     # Update global scope
#     global_scope['enc'] = enc_new
#     global_scope['db_file'] = db_file + '.new'

#     # Create engine
#     engine = get_engine()

#     # Restore db file
#     global_scope['db_file'] = db_file

#     return engine


def unlock():
    """
        Ask user to unlock the vault
    """

    # `False` = don't load menu after unlocking
    return menu.unlock(redirect_to_menu=False)
